package blatt10;

/**
 * Interface fuer den Router.
 * 
 * Dieses Interface darf nicht veraendert werden. Alle IP-Adressen, Netzmasken und
 * Gateways werden mit Hilfe des Datentypes Integer uebergeben.
 * 
 */
public interface IRouter {



	/**
	 * Setzt die lokale IP-Adresse.
	 * 
	 * @param localIP
	 */
	public void setLocalIP(int localIP);

	/**
	 * Gibt die Lokale IP-Adresse zurueck.
	 * 
	 * @return IP-Adresse
	 */
	public int getLocalIP();

	/**
	 * Fuegt eine Route hinzu.
	 * 
	 * Ist bereits eine Route mit dem gleichen Zielnetzwerk in der
	 * Routing-Tabelle, wird eine Fehlermeldung ausgegeben, zum Beispiel:
	 * "Failed to add route: Route already in use!"
	 * 
	 * Wurde die Route erfolgreich hinzugefuegt, muss dies ausgegeben.
	 * Weiterhin muss die Netzwerkadresse und die Broadcastadresse des Zielnetzes
	 * ausgegeben werden. Falls das Flag "H" gesetzt wurde, wird lediglich die
	 * Host-IP-Adresse ausgegeben.
	 * 
	 * @param destinationNetwork
	 * @param gateway
	 * @param genmask
	 * @param flags
	 * @param device
	 *            
	 */
	public void routeAdd(int destinationNetwork, int gateway, int genmask,
			Flags[] flags, String device);

	/**
	 * Loescht einen Eintrag aus der Routing-Tabelle.
	 * 
	 * Der Schluessel, um den Eintrag zu erkennen, ist das Zielnetzwerk. Am Ende
	 * muss ausgegeben werden, dass die Route geloescht wurde. Es muss ueberprueft 
	 * werden, ob die Subnetzmaske gueltig ist.
	 * 
	 * @param destinationNetwork
	 */
	public void routeDelete(int destinationNetwork);

	/**
	 * Modifiziert eine bestehende Route.
	 * 
	 * Der Schluessel, um das Netzwerk zu identifizieren, ist wiederum das
	 * Zielnetzwerk. Diese Methode funktioniert genau wie routeAdd, mit dem
	 * Unterschied, dass das davon ausgegangen wird, dass bereits eine Route mit
	 * dem gleichen Zielnetzwerk vorhanden ist. Dieses wird durch das neue
	 * ersetzt.
	 * 
	 * Ist das Zielnetzwerk noch nicht in der Routig-Tabelle eingetragen, muss 
	 * eine Fehlermeldung ausgegeben werden.
	 * 
	 * Wie bei routeAdd soll beim modifizieren wieder Netzwerkadresse und 
	 * Broadcastadresse des neuen Zielnetzwerkes ausgegeben werden.
	 * 
	 * @param destinationNetwork
	 * @param gateway
	 * @param genmask
	 * @param flags
	 * @param device
	 */
	public void routeModify(int destinationNetwork, int gateway, int genmask,
			Flags[] flags, String device);

	/**
	 * Sucht fuer eine Zieladresse die Passende Route.
	 * 
	 * Diese Methode bekommt eine IP-Adresse uebergeben und sucht aus der
	 * Routing-Tabbelle die passende Route raus. Der Rueckgabewert ist das
	 * Interface, ueber das geroutet werden soll.
	 * 
	 * Ist die Zieladresse der eigene Rechner, wird der String "localhost"
	 * zurueckgegegben. Auf der Konsole wird dann ausgegeben, dass der eigene
	 * Rechner das Ziel ist.
	 * 
	 * Kann keine passende Route gefunden werden wird der String "no route to host"
	 * zurueckgegeben und eine Fehlermeldung ausgegeben.
	 * 
	 * Achten sie auf die korrekten Flags.
	 * 
	 * @param destination
	 * @return Das Interface (das Device)
	 */
	public String findRoute(int destination);

	/**
	 * Druckt die Routing-Tabelle aus.
	 * 
	 * Die Ausgabe muss aehnlich sein, wie die von netstat -nr. Achten sie auch
	 * korrekte Einrückung. Eine exemplarische Ausgabe könnte so aussehen:

Destination     Gateway         Genmask         Flags           Iface           
212.69.12.0     0.0.0.0         255.255.255.0   U               eth0            
...

	 */
	public void printTable();

}
